/* Copyright (c) 2000-2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

/*
 * This file contains the functions that launch new windows such as
 * the help system, Rx, etc.
 *
 */

if (typeof(ko) == 'undefined') {
    var ko = {};
}
ko.launch = {};
ko.help = {};


(function () {
var _log = ko.logging.getLogger("ko.help");

/* XXX duplicated from help/content/contextHelp.js.  We do NOT want
   alwaysRaised attribute on the window, that's obnoxious! */

function openHelp(topic, contentPack)
{
  var helpFileURI = contentPack || helpFileURI;

  var topWindow = locateHelpWindow(helpFileURI);

  if ( topWindow ) {
    topWindow.focus();
    topWindow.displayTopic(topic);
  } else {
    const params = Components.classes["@mozilla.org/embedcomp/dialogparam;1"]
                             .createInstance(Components.interfaces.nsIDialogParamBlock);
    params.SetNumberStrings(2);
    params.SetString(0, helpFileURI);
    params.SetString(1, topic);
    ko.windowManager.openOrFocusDialog(
        "chrome://help/content/help.xul",
        "mozilla:help",
        "chrome,all,close=yes",
        params);
  }
}

function locateHelpWindow(contentPack) {
    const windowManagerInterface = Components
        .classes['@mozilla.org/appshell/window-mediator;1'].getService()
        .QueryInterface(Components.interfaces.nsIWindowMediator);
    const iterator = windowManagerInterface.getEnumerator("mozilla:help");
    var topWindow = null;
    var aWindow;

    while (iterator.hasMoreElements()) {
        aWindow = iterator.getNext();
        if (aWindow.getHelpFileURI() == contentPack) {
            topWindow = aWindow;
        }
    }
    return topWindow;
}
/* end of contextHelp.js duplication */

/**
 * open
 *
 * open Komodo help window
 *
 * @param {String} page  a page tag as defined in toc.xml
 */
this.open = function(page) {
    openHelp(page, 'chrome://komododoc/locale/komodohelp.rdf');
}

/**
 * getTag
 *
 * old help open method, DEPRECATED
 */
this.DEPRECATED_getTag = function(tag) {
    _log.warn("ko.help.getTag DEPRECATED, use ko.help.open");
    return tag;
}

/**
 * openTag
 *
 * old help open method, DEPRECATED
 */
this.DEPRECATED_openTag = function(tag) {
    _log.warn("ko.help.openTag DEPRECATED, use ko.help.open");
    ko.help.open(tag);
}

/**
 * tutorialProject
 *
 * used by the help system to open KPF files associated with a tutorial
 *
 * @param {String} tutorial   name of tutorial to open
 */
this.tutorialProject = function(tutorial)
{
    _log.info("tutorialProject");
    try {
        var koDirSvc = Components.classes["@activestate.com/koDirs;1"].
                getService(Components.interfaces.koIDirs);
        var osPathSvc = Components.classes["@activestate.com/koOsPath;1"].
                getService(Components.interfaces.koIOsPath);
        var sampleKpfPath = osPathSvc.joinlist(4,
                [koDirSvc.userDataDir, "samples", tutorial+"_tutorials", tutorial+"_tutorial.kpf"]);
        var response;
        if (! osPathSvc.exists(sampleKpfPath)) {
            response = ko.dialogs.okCancel(
                "The sample project could not be found at '"+
                    sampleKpfPath+"'. Your Komodo samples directory "+
                    "may be corrupt. Would you like Komodo to "+
                    "re-generate the samples tree?",
                "Cancel");
            if (response == "OK") {
                var initSvc = Components.classes["@activestate.com/koInitService;1"].
                        getService(Components.interfaces.koIInitService);
                initSvc.installSamples(true);
                if (! osPathSvc.exists(sampleKpfPath)) {
                    ko.dialogs.alert("Failed to create sample project " + sampleKpfPath);
                    return;
                }
            } else {
                return;
            }
        }
        var sampleKpfUrl = ko.uriparse.pathToURI(sampleKpfPath);
        var proj = ko.projects.manager.getProjectByURL(sampleKpfUrl);
        if (!proj) {
            ko.projects.open(sampleKpfUrl);
        } else {
            ko.projects.manager.setCurrentProject(proj);
        }
    } catch (ex) {
        _log.error("tutorialProject error: "+ex);
    }
}

/**
 * tutorials
 *
 * Open the Tutorial in the help browser
 *    "tutorial" can optionally be used to specify a particular tutorial
 *        to start with. Currently valid values are "perl", "python",
 *        "php", "xslt", and null (to go to Tutorial home page).
 *
 * @param {String} tutorial   name of tutorial to open
 */
this.tutorials = function(tutorial /* =null */)
{
    if (!tutorial) tutorial = 'showcase';
    var page = tutorial+"_tutorial";
    if (tutorial != 'showcase') {
        ko.help.tutorialProject(tutorial);
    }
    ko.help.open(page);
}


/**
 * language
 *
 * open language specific help for the current buffer.
 */
this.language = function() {
    // Get the current document's language.
    var language = null;
    var view = ko.window.focusedView();
    if (!view) view = ko.views.manager.currentView;
    if (view != null) {
        if (view.document) {
            language = view.document.subLanguage;
            if (language == "XML") {
                // use the primary language, not the sublanguage
                language = view.document.language
            }
        } else {
            language = view.language;
        }
    }

    // Get the help command appropriate for that language.
    var command=null, name=null;
    if (language) {
        if (gPrefs.hasStringPref(language+'HelpCommand')) {
            command = gPrefs.getStringPref(language+'HelpCommand');
        } else {
            // try to get from the language service
            var langRegistrySvc = Components.classes['@activestate.com/koLanguageRegistryService;1'].
                              getService(Components.interfaces.koILanguageRegistryService);
            var languageObj = langRegistrySvc.getLanguage(language);
            if (languageObj.searchURL) {



                command = "%(browser) "+languageObj.searchURL;

            }
        }
        if (command) {
            name = language + " Help";
        }
    }
    if (!command) {
        // Couldn't find language-specific help command -- use the default one.
        command = gPrefs.getStringPref('DefaultHelpCommand');
        name = "Help";
    }

    ko.run.runCommand(window,
                   command,
                   null, // cwd
                   null, // env
                   false, // insertOutput
                   false, // operateOnSelection
                   true, // doNotOpenOutputWindow
                   'no-console', // runIn
                   0, // parseOutput
                   '', // parseRegex
                   0, // showParsedOutputList
                   name); // name
}


/**
 * alternate
 *
 * uses the alternate help preference
 */
this.alternate = function() {
    var command = gPrefs.getStringPref('OtherHelpCommand');
    ko.run.runCommand(window,
                   command,
                   null, // cwd
                   null, // env
                   false, // insertOutput
                   false, // operateOnSelection
                   true, // doNotOpenOutputWindow
                   'no-console', // runIn
                   0, // parseOutput
                   '', // parseRegex
                   0, // showParsedOutputList
                   "Alternate Help"); // name
}

}).apply(ko.help);

(function () {
var _log = ko.logging.getLogger("ko.launch");



// Used for passing information reliably to the find dialog.
this.find2_dialog_args = null;

/**
 * Open the Find dialog.
 *
 * @param pattern {String} The pattern to search for.
 */
this.find = function(pattern /* =null */) {
    // Transfer focus to the hidden input buffer to capture keystrokes
    // from the user while find2.xul is loading. The find dialog will
    // retrieve these contents when it is ready.
    ko.inputBuffer.start();

    // Special global to pass info to find2.xul. Passing in via
    // openDialog() doesn't work if the dialog is already up.
    ko.launch.find2_dialog_args = {
        "pattern": pattern,
        "mode": "find"
    };

    // WARNING: Do NOT use ko.windowManager.openOrFocusDialog() here.
    // Because that ends up causing problems when re-opening the find
    // dialog (i.e. Ctrl+F when the Find dialog is already up).
    // openOrFocusDialog() results in the onfocus event but not onload
    // to the find dialog. That *could* be worked around with an onfocus
    // handler on find2.xul, but then you run into problems attempting
    // to focus the pattern textbox. (Or at least I did when experimenting
    // on Windows.)
    return ko.windowManager.openDialog(
        "chrome://komodo/content/find/find2.xul",
        "komodo_find2",
        "chrome,close=yes");
}


/**
 * Open the Find/Replace dialog.
 *
 * @param pattern {String} The pattern to search for.
 * @param repl {String} The replacement pattern.
 */
this.replace = function(pattern /* =null */, repl /* =null */) {
    // Transfer focus to the hidden input buffer to capture keystrokes
    // from the user while find2.xul is loading. The find dialog will
    // retrieve these contents when it is ready.
    ko.inputBuffer.start();

    // Special global to pass info to find2.xul. Passing in via
    // openDialog() doesn't work if the dialog is already up.
    ko.launch.find2_dialog_args = {
        "pattern": pattern,
        "repl": repl,
        "mode": "replace"
    };

    // WARNING: Do NOT use ko.windowManager.openOrFocusDialog() here.
    // (See above for why.)
    return ko.windowManager.openDialog(
        "chrome://komodo/content/find/find2.xul",
        "komodo_find2",
        "chrome,close=yes");
}

/**
 * Open the find dialog for searching in a "collection" find context.
 *
 * @param collection {koICollectionFindContext} defines in what to search.
 * @param pattern {string} is the pattern to search for. Optional.
 */
this.findInCollection = function(collection, pattern /* =null */) {
    // Transfer focus to the hidden input buffer to capture keystrokes
    // from the user while find2.xul is loading. The find dialog will
    // retrieve these contents when it is ready.
    ko.inputBuffer.start();

    // Special global to pass info to find2.xul. Passing in via
    // openDialog() doesn't work if the dialog is already up.
    ko.launch.find2_dialog_args = {
        "collection": collection,
        "pattern": pattern,
        "mode": "findincollection"
    };

    // WARNING: Do NOT use ko.windowManager.openOrFocusDialog() here.
    // (See above for why.)
    return ko.windowManager.openDialog(
        "chrome://komodo/content/find/find2.xul",
        "komodo_find2",
        "chrome,close=yes");
}

/**
 * Open the find dialog to find & replace in a "collection" of files.
 *
 * @param collection {koICollectionFindContext} defines in what to search.
 * @param pattern {String} The pattern to search for.
 * @param repl {String} The replacement pattern.
 */
this.replaceInCollection = function(collection, pattern /* =null */,
                                    repl /* =null */) {
    // Transfer focus to the hidden input buffer to capture keystrokes
    // from the user while find2.xul is loading. The find dialog will
    // retrieve these contents when it is ready.
    ko.inputBuffer.start();

    // Special global to pass info to find2.xul. Passing in via
    // openDialog() doesn't work if the dialog is already up.
    ko.launch.find2_dialog_args = {
        "collection": collection,
        "pattern": pattern,
        "repl": repl,
        "mode": "replaceincollection"
    };

    // WARNING: Do NOT use ko.windowManager.openOrFocusDialog() here.
    // (See above for why.)
    return ko.windowManager.openDialog(
        "chrome://komodo/content/find/find2.xul",
        "komodo_find2",
        "chrome,close=yes");
}

/**
 * Open Find dialog to search in the current project.
 *
 * @param pattern {String}
 */
this.findInCurrProject = function(pattern /* =null */) {
    // Transfer focus to the hidden input buffer to capture keystrokes
    // from the user while find2.xul is loading. The find dialog will
    // retrieve these contents when it is ready.
    ko.inputBuffer.start();

    // Special global to pass info to find2.xul. Passing in via
    // openDialog() doesn't work if the dialog is already up.
    ko.launch.find2_dialog_args = {
        "pattern": pattern,
        "mode": "findincurrproject"
    };

    // WARNING: Do NOT use ko.windowManager.openOrFocusDialog() here.
    // (See above for why.)
    return ko.windowManager.openDialog(
        "chrome://komodo/content/find/find2.xul",
        "komodo_find2",
        "chrome,close=yes");
}


/**
 * Open Find dialog to find & replace in the current project.
 *
 * @param pattern {String} The pattern to search for.
 * @param repl {String} The replacement pattern.
 */
this.replaceInCurrProject = function(pattern /* =null */, repl /* =null */) {
    // Transfer focus to the hidden input buffer to capture keystrokes
    // from the user while find2.xul is loading. The find dialog will
    // retrieve these contents when it is ready.
    ko.inputBuffer.start();

    // Special global to pass info to find2.xul. Passing in via
    // openDialog() doesn't work if the dialog is already up.
    ko.launch.find2_dialog_args = {
        "pattern": pattern,
        "repl": repl,
        "mode": "replaceincurrproject"
    };

    // WARNING: Do NOT use ko.windowManager.openOrFocusDialog() here.
    // (See above for why.)
    return ko.windowManager.openDialog(
        "chrome://komodo/content/find/find2.xul",
        "komodo_find2",
        "chrome,close=yes");
}

/**
 * Open Find dialog to search in files.
 *
 * @param pattern {String}
 * @param dirs {String}
 * @param includes {Array}
 * @param excludes {Array}
 */
this.findInFiles = function(pattern /* =null */, dirs /* =null */,
                            includes /* =null */, excludes /* =null */) {
    // Transfer focus to the hidden input buffer to capture keystrokes
    // from the user while find2.xul is loading. The find dialog will
    // retrieve these contents when it is ready.
    ko.inputBuffer.start();

    // Use the current view's cwd for interpreting relative paths.
    var view = ko.views.manager.currentView;
    var cwd = null;
    if (view != null &&
        view.getAttribute("type") == "editor" &&
        view.document.file &&
        view.document.file.isLocal) {
        cwd = view.document.file.dirName;
    }

    // Special global to pass info to find2.xul. Passing in via
    // openDialog() doesn't work if the dialog is already up.
    ko.launch.find2_dialog_args = {
        "pattern": pattern,
        "dirs": dirs,
        "includes": includes,
        "excludes": excludes,
        "cwd": cwd,
        "mode": "findinfiles"
    };

    // WARNING: Do NOT use ko.windowManager.openOrFocusDialog() here.
    // (See above for why.)
    return ko.windowManager.openDialog(
            "chrome://komodo/content/find/find2.xul",
            "komodo_find2",
            "chrome,close=yes");
}

/**
 * Open Find dialog to make replacements in files.
 *
 * @param pattern {String}
 * @param repl {String}
 * @param dirs {String}
 * @param includes {Array}
 * @param excludes {Array}
 */
this.replaceInFiles = function(pattern /* =null */, repl /* =null */,
                               dirs /* =null */, includes /* =null */,
                               excludes /* =null */) {
    // Transfer focus to the hidden input buffer to capture keystrokes
    // from the user while find2.xul is loading. The find dialog will
    // retrieve these contents when it is ready.
    ko.inputBuffer.start();

    // Special global to pass info to find2.xul. Passing in via
    // openDialog() doesn't work if the dialog is already up.
    ko.launch.find2_dialog_args = {
        "pattern": pattern,
        "repl": repl,
        "dirs": dirs,
        "includes": includes,
        "excludes": excludes,
        "mode": "replaceinfiles"
    };

    // WARNING: Do NOT use ko.windowManager.openOrFocusDialog() here.
    // (See above for why.)
    return ko.windowManager.openDialog(
            "chrome://komodo/content/find/find2.xul",
            "komodo_find2",
            "chrome,close=yes");
}


/**
 * runCommand
 *
 * open the run command dialog
 */
this.runCommand = function() {
    // Transfer focus to the hidden input buffer to capture keystrokes from the
    // user while run.xul is loading.  (Get the current view before calling
    // inputbuffer start so we have the correct focus coming out of the
    // dialog.)
    ko.inputBuffer.start();
    return window.openDialog("chrome://komodo/content/run/run.xul",
                      "_blank",
                      "chrome,close=yes");
}


/**
 * diff
 *
 * open the diff dialog, you must provide the diff
 *
 * @param {String} diff
 * @param {String} title
 * @param {String} message
 */
this.diff = function(diff, title /* ="Diff" */, message /* =null */)
{
    if (typeof(title) == "undefined") {
        title = "Diff";
    }
    if (typeof(message) == "undefined") {
        message = null;
    }

    var obj = new Object();
    obj.title = title;
    obj.diff = diff;
    obj.message = message;
    return ko.windowManager.openDialog(
        "chrome://komodo/content/dialogs/diff.xul",
        "_blank",
        "chrome,all,close=yes,resizable=yes,scrollbars=yes",
        obj);
}


/**
 * watchLocalFile
 *
 * prompt for a file to watch, then open a new watch window
 *
 */
this.watchLocalFile = function() {
    var filename = ko.filepicker.openFile();
    if (filename)
        return window.openDialog("chrome://komodo/content/tail/tail.xul",
                          "_blank",
                          "chrome,all,close=yes,resizable,scrollbars",
                          filename,
                          window);
    return null;
}


/**
 * Open the rx toolkit.
 *
 * @param mode {String} One of "match", "match-all", "split", "replace",
 *      "replace-all" or null for the default (match-all).
 * @param regex {String} The regular expression pattern, or null for
 *      the default (the last regex).
 * @param replacement {String}
 * @param searchText {String}
 * @param matchCase {Boolean} Whether to set the "I" match option.
 * @param isModal {Boolean} Whether to open the Rx dialog modally (default
 *      is false.
 * @param flagMultiline {Boolean} Whether to set the "M" match option.
 * @returns If 'isModal' this returns an object with 'language', 'mode',
 *      'regex', 'replacement', and 'searchText' attributes matching the
 *      closing state of the dialog.
 */
this.rxToolkit = function(mode /* =null */,
                          regex /* =null */,
                          replacement /* =null */,
                          searchText /* =null */,
                          matchCase /* =null */,
                          isModal /* =false */,
                          flagMultiline /* =null */)
{
    if (typeof(mode) == "undefined") { mode = null; }
    if (typeof(regex) == "undefined") { regex = null; }
    if (typeof(replacement) == "undefined") { replacement = null; }
    if (typeof(searchText) == "undefined") { searchText = null; }
    if (typeof(matchCase) == "undefined") { matchCase = null; }
    if (typeof(isModal) == "undefined") { isModal = false; }
    if (typeof(flagMultiline) == "undefined") { flagMultiline = null; }

    var args = {
        'mode': mode,
        'regex': regex,
        'replacement': replacement,
        'searchText': searchText,
        'matchCase': matchCase,
        'flagMultiline': flagMultiline
    };
    
    var openOptions = "chrome,all,close=yes,resizable,dependent=no";
    if (isModal) {
        openOptions += ",modal";
    }
    ko.windowManager.openOrFocusDialog("chrome://komodo/content/rxx/rxx.xul",
                      "komodo_rxx",
                      openOptions,
                      args);
    return args;
}




/**
 * httpInspector
 *
 * open the HTTPInspector window
 *
 */
this.httpInspector = function() {
    return ko.windowManager.openOrFocusDialog("chrome://komodo/content/httpinspector/httpInspector.xul",
                      "komodo_httpinspector",
                      "chrome,all,close=yes,resizable,dependent=no");
}


/**
 * openAddonsMgr
 *
 * open the extension/add ons manager window
 *
 */
this.openAddonsMgr = function launch_openAddonsMgr()
{
    return ko.windowManager.openOrFocusDialog("chrome://mozapps/content/extensions/extensions.xul",
                                       "Extension:Manager",
                                       "chrome,menubar,extra-chrome,toolbar,resizable");
                                       
}


/**
 * Opens the update manager and checks for updates to the application.
 * From http://plow/source/xref/mozilla/1.8/browser/base/content/utilityOverlay.js#452
 */
this.checkForUpdates = function checkForUpdates()
{
    var um = Components.classes["@mozilla.org/updates/update-manager;1"].
        getService(Components.interfaces.nsIUpdateManager);
    var prompter = Components.classes["@mozilla.org/updates/update-prompt;1"].
        createInstance(Components.interfaces.nsIUpdatePrompt);

    // If there's an update ready to be applied, show the "Update Downloaded"
    // UI instead and let the user know they have to restart the browser for
    // the changes to be applied. 
    if (um.activeUpdate && um.activeUpdate.state == "pending") {
        prompter.showUpdateDownloaded(um.activeUpdate);
    } else {
        prompter.checkForUpdates();
    }
}

}).apply(ko.launch);

/**
 * Input buffering
 * When you need to capture user input while a slow XUL window is loading you
 * can use the input buffer. Usage:
 *  - in some JS code:
 *      ko.inputBuffer.start()
 *      // open XUL window
 *
 *  - in slow XUL window onload handler:
 *      var contents = ko.inputBuffer.finish();
 *      // use the contents somehow
 */
ko.inputBuffer = {};
(function() { // ko.inputBuffer

var _isActive = false;
this.id = "hidden-input-buffer";
this.start = function()
{
    _isActive = true;
    var inputBufferWidget = document.getElementById(ko.inputBuffer.id);
    inputBufferWidget.focus();
}

this.focus = function(event)
{
    // if it is not active the hidden input buffer should not have the focus
    if (!_isActive && ko.views.manager.currentView) {
        // This has to be in a timeout for the controllers to work right.
        window.setTimeout('ko.views.manager.currentView.setFocus();', 1)
    }
}


this.finish = function()
{
    // Return the contents of the input buffer and stop buffering.
    var inputBufferWidget = document.getElementById(ko.inputBuffer.id);
    var contents = inputBufferWidget.value;
    inputBufferWidget.value = "";

    _isActive = false;
    return contents;
}

}).apply(ko.inputBuffer);



// backwards compat api for ko.help
var launch_LanguageHelp = ko.help.language;
var launch_AlternateLanguageHelp = ko.help.alternate;
var launch_Tutorials = ko.help.tutorials;
var launch_TutorialProject = ko.help.tutorialProject;
var launch_MainHelpTag = ko.help.DEPRECATED_openTag;
var launch_getHelpTag = ko.help.DEPRECATED_getTag;
var launch_MainHelp = ko.help.open;

// backwards compat api for ko.launch
var InputBuffer_Start = ko.inputBuffer.start;
var InputBuffer_OnFocus = ko.inputBuffer.focus;
var InputBuffer_Finish = ko.inputBuffer.finish;

// XXX globals maintained to keep things working
var gFindDialogPanel = null;
var gFindSearchTerm = null;
var gFindInFilesCwd = null;
var gFindInFilesSearchTerm = null;
var gFindInFilesFolders = null;
var gFindInFilesFiletypes = null;


var launch_rx = ko.launch.rxToolkit;


var launch_HttpInspectorWindow = ko.launch.httpInspector;

var launch_openAddonsMgr = ko.launch.openAddonsMgr;
var launch_watchLocalFile = ko.launch.watchLocalFile;
var launch_DiffWindow = ko.launch.diff;
var launch_Find = ko.launch.find;
var launch_Replace = ko.launch.replace;
var launch_RunCommand = ko.launch.runCommand;
var launch_FindInFiles = ko.launch.findInFiles;
