

if (typeof(ko)=='undefined') {
    var ko = {};
}
if (typeof(ko.toolboxes)=='undefined') {
    ko.toolboxes = {};
}

(function() {
var _obSvc = Components.classes["@mozilla.org/observer-service;1"].
        getService(Components.interfaces.nsIObserverService);

function changelistManager() {
    try {
        toolboxBaseManager.apply(this);
    } catch (ex) {
        log.exception(ex, "error creating changelist manager:");
    }
}

var toolboxBaseManager = ko.toolboxes.toolboxBaseManager;
// The following two lines ensure proper inheritance (see Flanagan, p. 144).
changelistManager.prototype = new toolboxBaseManager();
changelistManager.prototype.constructor = changelistManager;

// This returns null if successful, otherwise it returns an error message.
changelistManager.prototype.init = function() {
    try {
        this.name = "Change List";
        var errmsg;

        var koDirs = Components.classes["@activestate.com/koDirs;1"].
                getService(Components.interfaces.koIDirs);
        var os = Components.classes["@activestate.com/koOs;1"].
                getService(Components.interfaces.koIOs);
        var stPath = os.path.join(koDirs.userDataDir, 'changelist.kpf');

        var err = toolboxBaseManager.prototype._init.apply(this,
                    [this.name, "changelistview", stPath]);
        _obSvc.addObserver(this, 'file_status', false);
        var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
                           getService(Components.interfaces.koILastErrorService);
        if (err == "could not create") {
            errmsg = "You have enabled Change List support but Komodo "+
                         "could not create a Change List file at '"+stPath+
                         "': "+lastErrorSvc.getLastErrorMessage()+
                         ".  You must ensure that the file exists or "+
                         "can be created.";
            return errmsg;
        } else if (err == "could not load") {
            errmsg = "Could not load the Change List file at '"+stPath+
                         "': "+lastErrorSvc.getLastErrorMessage();
            return errmsg;
        }
    } catch (e) {
        log.exception(e);
    }
    return null;
}

changelistManager.prototype.finalize = function() {
    _obSvc.removeObserver(this, 'file_status');
};

changelistManager.prototype.getDefaultChangelist = function(type) {
        var prettyName = type.toUpperCase();
        var title = 'Default Changes: '+prettyName;
        var part = this.findPartByTypeAttributeValue('changelist', 'name', title);
        if (!part) {
            part = ko.projects.addSimplePart(title, this.toolbox, 'changelist');
        }
        return part;
}

changelistManager.prototype.getChanglistUrls = function(changelist) {
    var urls = [];
    try {
        var children = new Array();
        changelist.getChildren(children, new Object());
        children = children.value;
    } catch(e) {
        return urls;
    }
    var child;
    for (var i = 0; i < children.length; i++) {
        child = children[i];
        if (child.url) {
            urls.push(child.url);
        } else {
            var c = this.getChanglistUrls(child);
            if (c.length > 0) {
                urls = urls.concat(c);
            }
        }
    }
    return urls;
    
}

changelistManager.prototype.editChangelist = function(changelist) {
    var prefs = changelist.prefset;

    var editCallback = function(changelist_args) {
        if (changelist_args.retval != 'OK') {
            // Nothing to do.
            return;
        }
        var name = changelist_args.changelistName;
        var description = changelist_args.message;
        var urls = changelist_args.urls;
        ko.toolboxes.scc.updateChangelist(changelist, name, description, urls);
    }

    var obj = new Object();
    obj.changelistName = changelist.name;
    obj.urls = this.getChanglistUrls(changelist);
    if (prefs.hasStringPref("changelist_description")) {
        obj.message = prefs.getStringPref("changelist_description");
    } else {
        obj.message = "";
    }
    var sccSvc = null;
    if (obj.urls.length > 0) {
        sccSvc = ko.scc.getServiceForUrl(obj.urls[0]);
    }
    obj.sccSvc = sccSvc;
    obj.type = "changelist";
    obj.callbackFn = editCallback;

    // Show the dialog.
    window.openDialog("chrome://komodo/content/scc/commit.xul",
                      "_blank",
                      "chrome,titlebar,resizable=yes",
                      obj);
}

changelistManager.prototype.updateChangelist = function(changelist, name,
                                                        description, urls)
{
    // Warn if the urls are already part of another non-default changelist.
    var i;
    var changelist_name;
    var conflicts = [];
    var existing_entry;
    for (i=0; i < urls.length; i++) {
        existing_entry = this.findPartByTypeAttributeValue('file', 'url', urls[i]);
        if (existing_entry && existing_entry.parent != changelist) {
            changelist_name = existing_entry.parent.name;
            if (changelist_name.substr(0, 16) != 'Default Changes:') {
                conflicts.push("Changelist '" + changelist_name + "' already contains url: " + urls[i]);
            }
        }
    }
    if (conflicts.length > 0) {
        var alert_message = "The following files are already in an existing " +
                            "changelist. Do you wish to continue anyway?";
        var response = ko.dialogs.yesNo(alert_message, "No",
                                        conflicts.join("\n"),
                                        "Changelist conflicts");
        if (response == "No") {
            return;
        }
    }

    // Save the changelist properties.
    changelist.name = name;
    changelist.prefset.setStringPref("changelist_description", description);

    // Remove all urls from changlist.
    var children = new Array();
    changelist.getChildren(children, new Object());
    children = children.value;
    for (i = 0; i < children.length; i++) {
        changelist.removeChild(children[i]);
    }

    // Add all the provided urls.
    for (i=0; i < urls.length; i++) {
        existing_entry = this.findPartByTypeAttributeValue('file', 'url', urls[i]);
        if (existing_entry) {
            changelist_name = existing_entry.parent.name;
            if (changelist_name.substr(0, 16) == 'Default Changes:') {
                existing_entry.parent.removeChild(existing_entry);
            }
        }
        ko.projects.addFileWithURL(urls[i], changelist);
    }
    // Refresh the whole changelist tree.
    this.viewMgr.view.refresh(changelist.parent);
}

changelistManager.prototype.observe = function(subject, topic, data) {
    if (topic == "file_status") {
        // if there is a file that has scc edit, then add it to our default
        // scc changelist
        var fSvc = Components.classes["@activestate.com/koFileService;1"]
                .getService(Components.interfaces.koIFileService);
        var files = data.split("\n");

        for (var i=0; i < files.length; i++) {
            // find out if SCC Edit flag is on, if so, add it to the default
            // change list for the SCC type
            var file = fSvc.getFileFromURI(files[i]);
            if (!file.sccType) continue;
            //dump("file "+files[i]+" action: ["+file.sccAction+"]\n");
            // find the default changelist

            var entry = this.findPartByTypeAttributeValue('file', 'url', files[i]);
            if (file.sccAction) {
                if (entry) continue;
                ko.projects.addFileWithURL(files[i], this.getDefaultChangelist(file.sccType));
            } else {
                if (entry && entry.parent == this.getDefaultChangelist(file.sccType)) {
                    entry.parent.removeChild(entry);
                    this.viewMgr.view.refresh(entry.parent);
                }
            }
        }
    }
}


var onloadListener = {
    observe: function(subject, topic, data) {
        ko.toolboxes.scc = new changelistManager();
        ko.toolboxes.scc.init();
        ko.main.addWillQuitHandler(ko.toolboxes.scc.finalize, ko.toolboxes.scc);
    }
}
_obSvc.addObserver(onloadListener, "komodo-ui-started",false);

}).apply(ko.toolboxes);
